@@ -43,6 +43,10 @@ gboolean destroy_event(gpointer object, GdkEventAny *event, gpointer data)
 		gtk_timeout_remove(Xdialog.timer);
 		Xdialog.timer = 0;
 	}
+	if (Xdialog.timer2 != 0) {
+		gtk_timeout_remove(Xdialog.timer2);
+		Xdialog.timer2 = 0;
+	}
 	gtk_main_quit();
 	Xdialog.window = Xdialog.widget1 = Xdialog.widget2 = Xdialog.widget3 = NULL;
 
@@ -119,6 +123,13 @@ gboolean checked(GtkObject *button, gpointer data)
 	return TRUE;
 }
 
+gboolean timeout_exit(gpointer data)
+{
+	Xdialog.exit_code = 255;
+	gtk_widget_destroy(Xdialog.window);
+	return FALSE;
+}
+
 /* This function is called within the timeout functions so to force
  * the processing of all queued GTK events.
  */
@@ -307,6 +318,44 @@ gboolean progress_timeout(gpointer data)
 
 /* tailbox and logbox callbacks */
 
+#ifdef USE_GTK2
+gboolean tailbox_timeout(gpointer data)
+{
+	gchar buffer[1024];
+	int nchars = 0;
+	GtkTextIter end_iter;
+
+	GtkTextBuffer *text_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(Xdialog.widget1));
+
+	do {
+		if (!empty_gtk_queue() && (Xdialog.file_init_size <= 0))
+			return FALSE;
+
+		nchars = fread(buffer, sizeof(gchar), 1024, Xdialog.file);
+		if (nchars == 0)
+			break;
+
+		gtk_text_buffer_get_end_iter(text_buffer, &end_iter);
+		gtk_text_buffer_insert(text_buffer, &end_iter, buffer, nchars);
+
+		if (Xdialog.file_init_size > 0) 
+			Xdialog.file_init_size -= nchars;
+	} while (nchars == 1024);
+
+	if (nchars > 0) {
+		GtkTextMark *mark;
+		gtk_text_buffer_get_end_iter(text_buffer, &end_iter);
+
+		mark = gtk_text_buffer_create_mark(text_buffer, "end",
+			&end_iter, TRUE);
+
+		gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(Xdialog.widget1),
+			mark, 0, FALSE, 0, 0);
+	}
+
+	return TRUE;
+}
+#else
 gboolean tailbox_timeout(gpointer data)
 {
 	GtkAdjustment *adj;
@@ -351,6 +400,7 @@ gboolean tailbox_timeout(gpointer data)
 
 	return TRUE;
 }
+#endif
 
 gint tailbox_keypress(GtkObject *text, GdkEventKey *event,
 		      gpointer data)
@@ -625,11 +675,11 @@ gboolean hide_passwords(GtkObject *button, gpointer data)
 
 	visible = (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button)) == FALSE);
 
-	if (Xdialog.passwd >= entries)
+	if ((Xdialog.passwd < 10 && Xdialog.passwd >= entries) || Xdialog.passwd == 11)
 		gtk_entry_set_visibility(GTK_ENTRY(Xdialog.widget1), visible);
-	if (Xdialog.passwd >= entries-1 && entries > 1)
+	if (entries > 1 && ((Xdialog.passwd < 10 && Xdialog.passwd >= entries-1) || Xdialog.passwd == 12))
 		gtk_entry_set_visibility(GTK_ENTRY(Xdialog.widget2), visible);
-	if (Xdialog.passwd > 0 && entries > 2)
+	if (entries > 2 && ((Xdialog.passwd < 10 && Xdialog.passwd > 0) || Xdialog.passwd == 13))
 		gtk_entry_set_visibility(GTK_ENTRY(Xdialog.widget3), visible);
 
 	return TRUE;
@@ -637,6 +687,19 @@ gboolean hide_passwords(GtkObject *button, gpointer data)
 
 /* editbox callback */
 
+#ifdef USE_GTK2
+gboolean editbox_ok(gpointer object, gpointer data)
+{
+	GtkTextIter start_iter, end_iter;
+
+	GtkTextBuffer *text_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(Xdialog.widget1));
+	gtk_text_buffer_get_bounds(text_buffer, &start_iter, &end_iter);
+	fputs(gtk_text_buffer_get_text(text_buffer, &start_iter, &end_iter,
+		FALSE), Xdialog.output);
+
+	return TRUE;
+}
+#else
 gboolean editbox_ok(gpointer object, gpointer data)
 {
 	int length, i;
@@ -648,9 +711,39 @@ gboolean editbox_ok(gpointer object, gpointer data)
 
 	return TRUE;
 }
+#endif
 
 /* The print button callback (used by editbox, textbox and tailbox) */
 
+#ifdef USE_GTK2
+gboolean print_text(gpointer object, gpointer data)
+{
+	gint length;
+	char cmd[MAX_PRTCMD_LENGTH];
+	FILE * temp;
+	GtkTextIter start_iter, end_iter;
+	GtkTextBuffer *text_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(Xdialog.widget1));
+	
+	gtk_text_buffer_get_bounds(text_buffer, &start_iter, &end_iter);
+
+	strcpysafe(cmd, PRINTER_CMD, MAX_PRTCMD_LENGTH);
+	if (strlen(Xdialog.printer) != 0) {
+		strcatsafe(cmd, " "PRINTER_CMD_OPTION, MAX_PRTCMD_LENGTH);
+		strcatsafe(cmd, Xdialog.printer, MAX_PRTCMD_LENGTH);
+	}
+
+	length = gtk_text_buffer_get_char_count(text_buffer);
+
+	temp = popen(cmd, "w");
+	if (temp != NULL) {
+		fwrite(gtk_text_buffer_get_text(text_buffer, &start_iter,
+			&end_iter, FALSE), sizeof(gchar), length, temp);
+		pclose(temp);
+	}
+
+	return TRUE;
+}
+#else
 gboolean print_text(gpointer object, gpointer data)
 {
 	int length, i;
@@ -678,6 +771,7 @@ gboolean print_text(gpointer object, gpointer data)
 
 	return TRUE;
 }
+#endif
 
 /* rangebox callbacks */
 
@@ -815,6 +909,20 @@ gboolean print_selection(GtkButton *button, gpointer data)
 		return FALSE;
 }
 
+#ifdef USE_GTK2
+gboolean print_tree_selection(GtkButton *button, gpointer data)
+{
+	int i = 0;
+	for (i = 0 ; Xdialog.array[i].state != -1 ; i++) {
+		if (Xdialog.array[i].state > 0) {
+			fprintf(Xdialog.output, "%s\n", Xdialog.array[i].tag);
+ 			return TRUE;
+		}
+	}
+	return FALSE;
+}
+#endif
+
 gboolean menu_timeout(gpointer data)
 {
 	return print_selection(NULL, NULL);
@@ -829,18 +937,32 @@ gboolean move_to_row_timeout(gpointer data)
 	return FALSE;	
 }
 
-#if 0
-/* treeview callback */
-
-gboolean select_timeout(gpointer data)
+#ifdef USE_GTK2
+void cb_selection_changed(GtkObject *tree)
 {
-	gtk_tree_item_select(GTK_TREE_ITEM(Xdialog.array[Xdialog.array[0].state].widget));
+	GtkTreeIter tree_iter;
+	GtkTreeModel *model;
+	GtkTreeSelection* selection;
+	gchar *name, *tag;
+	int i = 0;
 
-	/* Run this timeout function only once ! */
-	return FALSE;	
+	model = gtk_tree_view_get_model(GTK_TREE_VIEW(Xdialog.widget1));
+	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(Xdialog.widget1));
+
+	if (gtk_tree_selection_get_selected(selection, &model, &tree_iter))
+		gtk_tree_model_get(model, &tree_iter, 0, &name, 1, &tag, -1);
+
+	for (i = 0 ; Xdialog.array[i].state != -1 ; i++) {
+		if (!strncmp(Xdialog.array[i].name, name, strlen(name)))
+			Xdialog.array[i].state = 1;
+		else
+			Xdialog.array[i].state = 0;
+	}
+
+	g_free(name);
+	g_free(tag);
 }
-#endif
-
+#else
 void cb_selection_changed(GtkObject *tree)
 {
 	GList *list;
@@ -858,12 +980,36 @@ void cb_selection_changed(GtkObject *tree)
 		}
 	}
 }
+#endif
 
 /* buildlist callbacks */
 
 /* the following function is to be called after each glist update so to
  * set the proper (in)sensitive status onto the Add/Remove buttons.
  */
+#ifdef USE_GTK2
+void sensitive_buttons(void)
+{
+	GtkTreeIter tree_iter;
+	GtkTreeModel *model;
+
+	model = gtk_tree_view_get_model(GTK_TREE_VIEW(Xdialog.widget1));
+
+	if (gtk_tree_model_get_iter_first(model, &tree_iter)) {
+		gtk_widget_set_sensitive(Xdialog.widget3, TRUE);
+	} else {
+		gtk_widget_set_sensitive(Xdialog.widget3, FALSE);
+	}
+
+	model = gtk_tree_view_get_model(GTK_TREE_VIEW(Xdialog.widget2));
+
+	if (gtk_tree_model_get_iter_first(model, &tree_iter)) {
+		gtk_widget_set_sensitive(Xdialog.widget4, TRUE);
+	} else {
+		gtk_widget_set_sensitive(Xdialog.widget4, FALSE);
+	}
+}
+#else
 void sensitive_buttons(void)
 {
 	gtk_widget_set_sensitive(Xdialog.widget3,
@@ -871,7 +1017,37 @@ void sensitive_buttons(void)
 	gtk_widget_set_sensitive(Xdialog.widget4,
 				 g_list_length(GTK_LIST(Xdialog.widget2)->children) != 0);
 }
+#endif
 
+#ifdef USE_GTK2
+gboolean add_to_list(GtkButton *button, gpointer data)
+{
+	GtkTreeIter tree_iter1, tree_iter2;
+	GtkTreeModel *model1, *model2;
+	GtkTreeSelection* selection1;
+
+	model1 = gtk_tree_view_get_model(GTK_TREE_VIEW(Xdialog.widget1));
+	selection1 = gtk_tree_view_get_selection(GTK_TREE_VIEW(Xdialog.widget1));
+	model2 = gtk_tree_view_get_model(GTK_TREE_VIEW(Xdialog.widget2));
+
+	if (gtk_tree_selection_get_selected(selection1, &model1, &tree_iter1)) {
+		gchar *name, *tag;
+		gtk_tree_model_get(model1, &tree_iter1, 0, &name, 1, &tag, -1);
+
+		gtk_list_store_append(GTK_LIST_STORE(model2), &tree_iter2);
+		gtk_list_store_set(GTK_LIST_STORE(model2), &tree_iter2, 0,
+			 name, 1, tag, -1);
+		gtk_list_store_remove(GTK_LIST_STORE(model1), &tree_iter1);
+
+		g_free(name);
+		g_free(tag);
+	}
+  
+	sensitive_buttons(); 
+
+	return TRUE;
+}
+#else
 gboolean add_to_list(GtkButton *button, gpointer data)
 {
 	GList *selected;
@@ -884,7 +1060,36 @@ gboolean add_to_list(GtkButton *button, gpointer data)
 
 	return TRUE;
 }
+#endif
 
+#ifdef USE_GTK2
+gboolean remove_from_list(GtkButton *button, gpointer data)
+{
+	GtkTreeIter tree_iter1, tree_iter2;
+	GtkTreeModel *model1, *model2;
+	GtkTreeSelection* selection2;
+
+	model1 = gtk_tree_view_get_model(GTK_TREE_VIEW(Xdialog.widget1));
+	model2 = gtk_tree_view_get_model(GTK_TREE_VIEW(Xdialog.widget2));
+	selection2 = gtk_tree_view_get_selection(GTK_TREE_VIEW(Xdialog.widget2));
+
+	if (gtk_tree_selection_get_selected(selection2, &model2, &tree_iter2)) {
+		gchar *name, *tag;
+		gtk_tree_model_get(model2, &tree_iter2, 0, &name, 1, &tag, -1);
+
+		gtk_list_store_append(GTK_LIST_STORE(model1), &tree_iter1);
+		gtk_list_store_set(GTK_LIST_STORE(model1), &tree_iter1, 0, name, 1, tag, -1);
+		gtk_list_store_remove(GTK_LIST_STORE(model2), &tree_iter2);
+
+		g_free(name);
+		g_free(tag);
+	}
+
+	sensitive_buttons();
+
+	return TRUE;
+}
+#else
 gboolean remove_from_list(GtkButton *button, gpointer data)
 {
 	GList *selected;
@@ -897,7 +1102,42 @@ gboolean remove_from_list(GtkButton *button, gpointer data)
 
 	return TRUE;
 }
+#endif
 
+#ifdef USE_GTK2
+gboolean print_list(GtkButton *button, gpointer data)
+{
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	gboolean valid;
+	gboolean flag = FALSE;
+
+	model = gtk_tree_view_get_model(GTK_TREE_VIEW(Xdialog.widget2));
+	valid = gtk_tree_model_get_iter_first(model, &iter);
+
+	while (valid) {
+		gchar *name_data, *tag_data;
+
+		gtk_tree_model_get(model, &iter, 0, &name_data, 1, &tag_data, -1);
+
+		if (flag)
+			fprintf(Xdialog.output, "%s", Xdialog.separator);
+
+		fprintf(Xdialog.output, "%s", tag_data);
+		flag = TRUE;
+
+		g_free(name_data);
+		g_free(tag_data);
+
+		valid = gtk_tree_model_iter_next(model, &iter);
+	}
+
+	if (flag)
+		fprintf(Xdialog.output, "\n");
+
+	return TRUE;
+}
+#else
 gboolean print_list(GtkButton *button, gpointer data)
 {
 	GList *children;
@@ -922,6 +1162,7 @@ gboolean print_list(GtkButton *button, gpointer data)
 
 	return TRUE;
 }
+#endif
 
 gboolean buildlist_timeout(gpointer data)
 {
