@@ -23,12 +23,14 @@
 #include "callbacks.h"
 #include "support.h"
 
+#ifndef USE_GTK2
 #include "yes.xpm"
 #include "no.xpm"
 #include "help.xpm"
 #include "print.xpm"
 #include "next.xpm"
 #include "previous.xpm"
+#endif
 
 /* Global structure and variables */
 extern Xdialog_data Xdialog;
@@ -364,15 +366,35 @@ static GtkWidget *set_button(gchar *default_text, gpointer buttonbox, gint event
 			     gboolean grab_default)
 {
 	GtkWidget *button;
+#ifdef USE_GTK2
+	gchar	  *stock_id = NULL;
+#else
 	GdkBitmap *mask;
 	GdkColor  transparent;
 	GdkPixmap *pixmap;
+	gchar	  **xpm = NULL;
+#endif
 	GtkWidget *icon;
 	GtkWidget *hbox;
 	GtkWidget *label;
-	gchar	  **xpm = NULL;
 	gchar     *text = default_text;
 
+#ifdef USE_GTK2
+	if (Xdialog.buttons_style != TEXT_ONLY) {
+		if (!strcmp(text, OK) || !strcmp(text, YES))
+			stock_id = "gtk-ok";
+		else if (!strcmp(text, CANCEL) || !strcmp(text, NO))
+			stock_id = "gtk-close";
+		else if (!strcmp(text, HELP))
+			stock_id = "gtk-help";
+		else if (!strcmp(text, PRINT))
+			stock_id = "gtk-print";
+		else if (!strcmp(text, NEXT) || !strcmp(text, ADD))
+			stock_id = "gtk-go-forward";
+		else if (!strcmp(text, PREVIOUS) || !strcmp(text, REMOVE))
+			stock_id = "gtk-go-back";
+	}
+#else
 	if (Xdialog.buttons_style != TEXT_ONLY) {
 		if (!strcmp(text, OK) || !strcmp(text, YES))
 			xpm = yes_xpm;
@@ -387,13 +409,14 @@ static GtkWidget *set_button(gchar *default_text, gpointer buttonbox, gint event
 		else if (!strcmp(text, PREVIOUS) || !strcmp(text, REMOVE))
 			xpm = previous_xpm;
 	}
+#endif
 
 	if (strlen(Xdialog.ok_label) != 0 && (!strcmp(text, OK) || !strcmp(text, YES)))
 		text = Xdialog.ok_label;
 	else if (strlen(Xdialog.cancel_label) != 0 && (!strcmp(text, CANCEL) || !strcmp(text, NO)))
 		text = Xdialog.cancel_label;
 
-	if (xpm == NULL)	/* buttons with text only */
+	if (Xdialog.buttons_style == TEXT_ONLY)
 		button = gtk_button_new_with_label(text);
 	else {			/* buttons with icons */
 		button = gtk_button_new();
@@ -401,11 +424,15 @@ static GtkWidget *set_button(gchar *default_text, gpointer buttonbox, gint event
 		gtk_container_add(GTK_CONTAINER(button), hbox);
 		gtk_widget_show(hbox);
 
+#ifdef USE_GTK2
+		icon = gtk_image_new_from_stock(stock_id, GTK_ICON_SIZE_BUTTON);
+#else
 		pixmap = gdk_pixmap_create_from_xpm_d(Xdialog.window->window,
 						      &mask, &transparent, xpm);
 		icon = gtk_pixmap_new(pixmap, mask);
 		gdk_pixmap_unref(pixmap);
 		gdk_pixmap_unref(mask);
+#endif
 		gtk_container_add(GTK_CONTAINER(hbox), icon);
 		gtk_widget_show(icon);
 
@@ -524,6 +551,11 @@ static GtkWidget *set_scrollable_text(void)
 	gtk_box_pack_start (GTK_BOX(Xdialog.vbox), scrollwin, TRUE, TRUE, 0);
 
 	text = gtk_text_view_new();
+#if 0
+	/* TODO: implement fixed font style support... */
+	if (Xdialog.fixed_font) {
+	}
+#endif
 	gtk_widget_show(text);
 	gtk_container_add(GTK_CONTAINER (scrollwin), text);
 	return text;
@@ -1162,7 +1194,8 @@ void create_inputbox(gchar *optarg, gchar *options[], gint entries)
 	} else
 		Xdialog.widget3 = NULL;
 
-	if (Xdialog.passwd > 0) {
+	if ((Xdialog.passwd > 0 && Xdialog.passwd < 10) ||
+            (Xdialog.passwd > 10 && Xdialog.passwd <= entries + 10)) {
 		hide_button = gtk_check_button_new_with_label(HIDE_TYPING);
 		gtk_box_pack_start(Xdialog.vbox, hide_button, TRUE, TRUE, 0);
 		gtk_widget_show(hide_button);
@@ -1729,12 +1762,12 @@ void create_treeview(gchar *optarg, gchar *options[], gint list_size)
 	GtkWidget *button_ok;
 	GtkWidget *item;
 	GtkWidget *selected = NULL;
-	GtkTree *origtree, *tree;
+	GtkTree *tree;
 	GtkTree *oldtree[MAX_TREE_DEPTH];
 	GtkWidget *subtree;
 	GtkTooltips *tooltips = NULL;
 	int depth = 0;
-	int i;
+	int level, i;
 	int params = 4 + Xdialog.tips;
 
 	if (Xdialog.tips == 1)
@@ -1756,10 +1789,10 @@ void create_treeview(gchar *optarg, gchar *options[], gint list_size)
 	button_ok = set_all_buttons(FALSE, TRUE);
 	gtk_signal_connect(GTK_OBJECT(button_ok), "clicked", GTK_SIGNAL_FUNC(print_selection), NULL);
 
-	tree = oldtree[0] = origtree = GTK_TREE(Xdialog.widget1);
+	tree = oldtree[0] = GTK_TREE(Xdialog.widget1);
 
-	gtk_tree_set_view_mode(origtree, GTK_TREE_VIEW_ITEM);
-	gtk_tree_set_selection_mode(origtree, GTK_SELECTION_BROWSE);
+	gtk_tree_set_view_mode(tree, GTK_TREE_VIEW_ITEM);
+	gtk_tree_set_selection_mode(tree, GTK_SELECTION_BROWSE);
 
 	for (i = 0 ; i < list_size ; i++) {
 		strcpysafe(Xdialog.array[i].tag, options[params*i], MAX_ITEM_LENGTH);
@@ -1767,44 +1800,45 @@ void create_treeview(gchar *optarg, gchar *options[], gint list_size)
 		item = gtk_tree_item_new_with_label(Xdialog.array[i].name);
 		Xdialog.array[i].widget = item;
 
-		if (i > 0 && atoi(options[params*i+3]) < atoi(options[params*(i-1)+3])) {
-			tree = oldtree[depth];
-			depth--;
+		level = atoi(options[params*i+3]);
+		if (i > 0) {
+			if (atoi(options[params*(i-1)+3]) > level) {
+				depth = level;
+				tree = oldtree[level];
+			}
 		}
-		if (atoi(options[params*i+3]) == 0)
-			tree = origtree;
 
 		gtk_tree_append(tree, item);
 
-		if (i+1 < list_size &&
-		    atoi(options[params*i+3]) < atoi(options[params*(i+1)+3])) {
-			if (atoi(options[params*i+3])+1 != atoi(options[params*(i+1)+3])) {
-				fprintf(stderr,
-					"Xdialog: You cannot increment the --treeview depth "\
-					"by more than one level each time !  Aborting...\n");
-					exit(255);
+		if (i+1 < list_size) {
+			if (level < atoi(options[params*(i+1)+3])) {
+				if (atoi(options[params*(i+1)+3]) != level + 1) {
+					fprintf(stderr,
+						"Xdialog: You cannot increment the --treeview depth "\
+						"by more than one level each time !  Aborting...\n");
+						exit(255);
+				}
+				subtree = gtk_tree_new();
+				gtk_signal_connect(GTK_OBJECT(subtree), "button_press_event",
+						   GTK_SIGNAL_FUNC(double_click_event),
+						   GTK_WIDGET(button_ok));
+				gtk_tree_item_set_subtree(GTK_TREE_ITEM(item), subtree);
+				depth++;
+				if (depth > MAX_TREE_DEPTH) {
+					fprintf(stderr,
+						"Xdialog: Max allowed depth for "\
+						"--treeview is %d !  Aborting...\n",
+						MAX_TREE_DEPTH);
+						exit(255);
+				}
+				tree = GTK_TREE(subtree);
+				oldtree[depth] = tree;
+				gtk_tree_set_selection_mode(tree, GTK_SELECTION_BROWSE);
 			}
-			subtree = gtk_tree_new();
-			gtk_signal_connect(GTK_OBJECT(subtree), "button_press_event",
-					   GTK_SIGNAL_FUNC(double_click_event),
-					   GTK_WIDGET(button_ok));
-			gtk_tree_item_set_subtree(GTK_TREE_ITEM(item), subtree);
-			depth++;
-			if (depth > MAX_TREE_DEPTH) {
-				fprintf(stderr,
-					"Xdialog: Max allowed depth for "\
-					"--treeview is %d !  Aborting...\n",
-					MAX_TREE_DEPTH);
-					exit(255);
-			}
-			oldtree[depth] = tree;
-			tree = GTK_TREE(subtree);
-			gtk_tree_set_selection_mode(tree, GTK_SELECTION_BROWSE);
 		}
 
 		if (item_status(item, options[params*i+2], Xdialog.array[i].tag) == 1 && selected == NULL) {
 			selected = item;
-/*			gtk_tree_item_select(GTK_TREE_ITEM(selected)); */
 			Xdialog.array[0].state = i;
 		}
 
@@ -1826,11 +1860,6 @@ void create_treeview(gchar *optarg, gchar *options[], gint list_size)
 		gtk_tree_item_select(GTK_TREE_ITEM(selected));
 #endif
 
-#if 0
-	if (selected != NULL)
-		gtk_timeout_add(1, select_timeout, NULL);
-#endif
-
 	gtk_signal_connect(GTK_OBJECT(Xdialog.widget1), "selection_changed",
 			   GTK_SIGNAL_FUNC(cb_selection_changed), Xdialog.widget1);
 	gtk_signal_connect(GTK_OBJECT(Xdialog.widget1), "button_press_event",
@@ -1867,9 +1896,17 @@ void create_filesel(gchar *optarg, gboolean dsel_flag)
 	gtk_file_selection_set_filename(filesel, optarg);
 	gtk_file_selection_complete(filesel, optarg);
 
-	/* If we want a directory selector, then hide the file list parent */
-	if (dsel_flag)
+	/* If we want a directory selector, then hide the file list parent and
+           the file list entry field. Also clear the file selection to erase
+           the auto-completed filename. Finally, disable the file operation
+           buttons to keep only the "make new directory" one. */
+	if (dsel_flag) {
 		gtk_widget_hide_all(GTK_WIDGET(GTK_WIDGET(filesel->file_list)->parent));
+		gtk_widget_hide(GTK_WIDGET(filesel->selection_entry));
+		gtk_file_selection_set_filename(filesel, "");
+		gtk_widget_set_sensitive(GTK_WIDGET(filesel->fileop_del_file), FALSE);
+		gtk_widget_set_sensitive(GTK_WIDGET(filesel->fileop_ren_file), FALSE);
+	}
 
 	/* Hide fileops buttons if requested */
 	if (!Xdialog.buttons || dialog_compat)
@@ -1934,7 +1971,7 @@ void create_filesel(gchar *optarg, gboolean dsel_flag)
 }
 
 
-void create_colorsel(gchar *optarg)
+void create_colorsel(gchar *optarg, gdouble *colors)
 {
 	GtkColorSelectionDialog *colorsel;
 	GtkWidget *box;
@@ -1955,6 +1992,11 @@ void create_colorsel(gchar *optarg)
 	Xdialog.vbox = GTK_BOX(colorsel->main_vbox);
 #endif
 
+	/* We must realize the widget before moving it and creating the icon and
+           buttons pixmaps...
+        */
+	gtk_widget_realize(Xdialog.window);
+
 	/* Set the text */
 	box = gtk_widget_get_ancestor(set_label(optarg, TRUE), gtk_box_get_type());
 	gtk_box_reorder_child(Xdialog.vbox, box, 0);
@@ -1965,9 +2007,6 @@ void create_colorsel(gchar *optarg)
 	/* If requested, add a check button into the colorsel action area */
 	set_check_button(GTK_WIDGET(Xdialog.vbox));
 
-	/* We must realize the widget before moving it and creating the buttons pixmaps */
-	gtk_widget_realize(Xdialog.window);
-
 	/* Set the window size and placement policy */
 	set_window_size_and_placement();
 
@@ -2003,6 +2042,8 @@ void create_colorsel(gchar *optarg)
 	if (Xdialog.help)
 		set_button(HELP, hbuttonbox, 2, FALSE);
 
+	gtk_color_selection_set_color(GTK_COLOR_SELECTION(colorsel->colorsel), colors);
+
 	/* Setup callbacks */
 	gtk_signal_connect(GTK_OBJECT(Xdialog.window), "destroy",
 			   GTK_SIGNAL_FUNC(destroy_event), NULL);
